﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("XCSAfterCrossoverOperator", "Description missing")]
  [StorableClass]
  public class XCSAfterCrossoverOperator : SingleSuccessorOperator {

    #region Parameter Properties
    public IValueLookupParameter<IntValue> NumerosityParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["Numerosity"]; }
    }
    public IValueLookupParameter<IntValue> ExperienceParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["Experience"]; }
    }
    public IValueLookupParameter<IntValue> TimestampParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["Timestamp"]; }
    }
    public IValueLookupParameter<DoubleValue> PredictionParameter {
      get { return (IValueLookupParameter<DoubleValue>)Parameters["Prediction"]; }
    }
    public IValueLookupParameter<DoubleValue> AverageActionSetSizeParameter {
      get { return (IValueLookupParameter<DoubleValue>)Parameters["AverageActionSetSize"]; }
    }
    public IValueLookupParameter<DoubleValue> ErrorParameter {
      get { return (IValueLookupParameter<DoubleValue>)Parameters["Error"]; }
    }
    public IValueLookupParameter<DoubleValue> FitnessParameter {
      get { return (IValueLookupParameter<DoubleValue>)Parameters["Fitness"]; }
    }
    public IValueLookupParameter<BoolValue> InsertInPopulationParameter {
      get { return (IValueLookupParameter<BoolValue>)Parameters["InsertInPopulation"]; }
    }
    public IValueLookupParameter<IntValue> TempIDParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["TempID"]; }
    }
    public ILookupParameter<IntValue> CurrentIterationParameter {
      get { return (ILookupParameter<IntValue>)Parameters["CurrentIteration"]; }
    }
    public ILookupParameter<ItemArray<DoubleValue>> ParentAverageActionSetSizeParameter {
      get { return (ILookupParameter<ItemArray<DoubleValue>>)Parameters["ParentAverageActionSetSize"]; }
    }
    public ILookupParameter<ItemArray<DoubleValue>> ParentPredictionParameter {
      get { return (ILookupParameter<ItemArray<DoubleValue>>)Parameters["ParentPrediction"]; }
    }
    public ILookupParameter<ItemArray<DoubleValue>> ParentErrorParameter {
      get { return (ILookupParameter<ItemArray<DoubleValue>>)Parameters["ParentError"]; }
    }
    public ILookupParameter<ItemArray<DoubleValue>> ParentFitnessParameter {
      get { return (ILookupParameter<ItemArray<DoubleValue>>)Parameters["ParentFitness"]; }
    }
    #endregion

    [StorableConstructor]
    protected XCSAfterCrossoverOperator(bool deserializing) : base(deserializing) { }
    protected XCSAfterCrossoverOperator(XCSAfterCrossoverOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public XCSAfterCrossoverOperator()
      : base() {
      Parameters.Add(new ValueLookupParameter<IntValue>("Numerosity"));
      Parameters.Add(new ValueLookupParameter<IntValue>("Experience"));
      Parameters.Add(new ValueLookupParameter<IntValue>("Timestamp"));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("AverageActionSetSize"));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("Prediction"));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("Error"));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("Fitness"));
      Parameters.Add(new ValueLookupParameter<BoolValue>("InsertInPopulation"));
      Parameters.Add(new ValueLookupParameter<IntValue>("TempID"));
      Parameters.Add(new LookupParameter<IntValue>("CurrentIteration"));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("ParentAverageActionSetSize"));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("ParentPrediction"));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("ParentError"));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("ParentFitness"));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new XCSAfterCrossoverOperator(this, cloner);
    }

    public override IOperation Apply() {
      NumerosityParameter.ActualValue = new IntValue(1);
      ExperienceParameter.ActualValue = new IntValue(0);
      TimestampParameter.ActualValue = new IntValue(CurrentIterationParameter.ActualValue.Value);
      var parentAverageActionSetSize = ParentAverageActionSetSizeParameter.ActualValue;
      var parentPrecision = ParentPredictionParameter.ActualValue;
      var parentPrecisionError = ParentErrorParameter.ActualValue;
      var parentFitness = ParentFitnessParameter.ActualValue;
      AverageActionSetSizeParameter.ActualValue = new DoubleValue(parentAverageActionSetSize.Select(x => x.Value).Average());
      PredictionParameter.ActualValue = new DoubleValue(parentPrecision.Select(x => x.Value).Average());
      ErrorParameter.ActualValue = new DoubleValue(0.25 * parentPrecisionError.Select(x => x.Value).Average());
      FitnessParameter.ActualValue = new DoubleValue(0.1 * parentFitness.Select(x => x.Value).Average());
      InsertInPopulationParameter.ActualValue = new BoolValue(true);
      TempIDParameter.ActualValue = new IntValue(-1);
      return base.Apply();
    }
  }
}
