﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.CombinedIntegerVectorEncoding;
using HeuristicLab.Encodings.ConditionActionEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Problems.ConditionActionClassification {
  [Item("CombinedIntegerVectorClassifierFetcher", "")]
  [StorableClass]
  public class CombinedIntegerVectorClassifierFetcher : SingleSuccessorOperator, IClassifierFetcher {
    #region parameters
    public IValueLookupParameter<IClassifier> CurrentClassifierToMatchParameter {
      get { return (IValueLookupParameter<IClassifier>)Parameters["CurrentClassifierToMatch"]; }
    }
    public ILookupParameter<IConditionActionProblemData> ProblemDataParameter {
      get { return (ILookupParameter<IConditionActionProblemData>)Parameters["ProblemData"]; }
    }
    public ILookupParameter<IntValue> ActionPartLengthParameter {
      get { return (ILookupParameter<IntValue>)Parameters["ActionPartLength"]; }
    }
    public ILookupParameter<IntMatrix> BoundsParameter {
      get { return (ILookupParameter<IntMatrix>)Parameters["Bounds"]; }
    }
    public ILookupParameter<IntValue> IterationParameter {
      get { return (ILookupParameter<IntValue>)Parameters["Iteration"]; }
    }
    #endregion

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CombinedIntegerVectorClassifierFetcher(this, cloner);
    }
    protected CombinedIntegerVectorClassifierFetcher(CombinedIntegerVectorClassifierFetcher original, Cloner cloner)
      : base(original, cloner) {
    }
    [StorableConstructor]
    protected CombinedIntegerVectorClassifierFetcher(bool deserializing) : base(deserializing) { }
    public CombinedIntegerVectorClassifierFetcher()
      : base() {
      Parameters.Add(new ValueLookupParameter<IClassifier>("CurrentClassifierToMatch"));
      Parameters.Add(new LookupParameter<IConditionActionProblemData>("ProblemData"));
      Parameters.Add(new LookupParameter<IntValue>("ActionPartLength"));
      Parameters.Add(new LookupParameter<IntMatrix>("Bounds"));
      Parameters.Add(new LookupParameter<IntValue>("Iteration"));
    }

    public override IOperation Apply() {
      Dataset dataset = ProblemDataParameter.ActualValue.Dataset;
      int row = IterationParameter.ActualValue.Value % dataset.Rows;
      int[] elements = new int[dataset.Columns];
      for (int i = 0; i < dataset.Columns; i++) {
        elements[i] = int.Parse(dataset.GetValue(row, i));
      }
      CurrentClassifierToMatchParameter.ActualValue = new CombinedIntegerVector(elements, ActionPartLengthParameter.ActualValue.Value, BoundsParameter.ActualValue);
      return base.Apply();
    }
  }
}
